
# Hashes for binaries used in Bitmask Bundle.
# This file has been automatically generated by `setup.py hash_binaries`
# DO NOT modify it manually.

OPENVPN_BIN = "e4e771603cf65fe8951011388f0e0c20872443774ef46bfb26631df9605e9033"
BITMASK_ROOT = "0c823ed7fb59a09405f1e0f800539ac39c0b09aff79c8a5a4561097e9dea736f"
