# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'src/leap/bitmask/gui/ui/password_change.ui'
#
# Created: Mon Dec 29 09:53:14 2014
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_PasswordChange(object):
    def setupUi(self, PasswordChange):
        PasswordChange.setObjectName("PasswordChange")
        PasswordChange.resize(459, 231)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(PasswordChange.sizePolicy().hasHeightForWidth())
        PasswordChange.setSizePolicy(sizePolicy)
        self.verticalLayout = QtGui.QVBoxLayout(PasswordChange)
        self.verticalLayout.setObjectName("verticalLayout")
        self.grid_layout = QtGui.QGridLayout()
        self.grid_layout.setObjectName("grid_layout")
        self.username_label = QtGui.QLabel(PasswordChange)
        self.username_label.setObjectName("username_label")
        self.grid_layout.addWidget(self.username_label, 0, 0, 1, 1)
        self.new_password_label = QtGui.QLabel(PasswordChange)
        self.new_password_label.setObjectName("new_password_label")
        self.grid_layout.addWidget(self.new_password_label, 3, 0, 1, 1)
        self.new_password_lineedit = QtGui.QLineEdit(PasswordChange)
        self.new_password_lineedit.setEchoMode(QtGui.QLineEdit.Password)
        self.new_password_lineedit.setObjectName("new_password_lineedit")
        self.grid_layout.addWidget(self.new_password_lineedit, 3, 1, 1, 1)
        self.new_password_confirmation_label = QtGui.QLabel(PasswordChange)
        self.new_password_confirmation_label.setObjectName("new_password_confirmation_label")
        self.grid_layout.addWidget(self.new_password_confirmation_label, 4, 0, 1, 1)
        self.current_password_label = QtGui.QLabel(PasswordChange)
        self.current_password_label.setObjectName("current_password_label")
        self.grid_layout.addWidget(self.current_password_label, 1, 0, 1, 1)
        self.new_password_confirmation_lineedit = QtGui.QLineEdit(PasswordChange)
        self.new_password_confirmation_lineedit.setEchoMode(QtGui.QLineEdit.Password)
        self.new_password_confirmation_lineedit.setObjectName("new_password_confirmation_lineedit")
        self.grid_layout.addWidget(self.new_password_confirmation_lineedit, 4, 1, 1, 1)
        self.current_password_lineedit = QtGui.QLineEdit(PasswordChange)
        self.current_password_lineedit.setEchoMode(QtGui.QLineEdit.Password)
        self.current_password_lineedit.setObjectName("current_password_lineedit")
        self.grid_layout.addWidget(self.current_password_lineedit, 1, 1, 1, 1)
        spacerItem = QtGui.QSpacerItem(20, 10, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.grid_layout.addItem(spacerItem, 2, 1, 1, 1)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.username_lineedit = QtGui.QLineEdit(PasswordChange)
        self.username_lineedit.setEnabled(False)
        self.username_lineedit.setObjectName("username_lineedit")
        self.horizontalLayout.addWidget(self.username_lineedit)
        self.grid_layout.addLayout(self.horizontalLayout, 0, 1, 1, 1)
        self.verticalLayout.addLayout(self.grid_layout)
        self.flash_label = QtGui.QLabel(PasswordChange)
        self.flash_label.setAlignment(QtCore.Qt.AlignCenter)
        self.flash_label.setObjectName("flash_label")
        self.verticalLayout.addWidget(self.flash_label)
        spacerItem1 = QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)
        self.button_layout = QtGui.QHBoxLayout()
        self.button_layout.setObjectName("button_layout")
        spacerItem2 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.button_layout.addItem(spacerItem2)
        self.cancel_button = QtGui.QPushButton(PasswordChange)
        self.cancel_button.setAutoDefault(False)
        self.cancel_button.setObjectName("cancel_button")
        self.button_layout.addWidget(self.cancel_button)
        self.ok_button = QtGui.QPushButton(PasswordChange)
        self.ok_button.setAutoDefault(False)
        self.ok_button.setDefault(True)
        self.ok_button.setObjectName("ok_button")
        self.button_layout.addWidget(self.ok_button)
        self.verticalLayout.addLayout(self.button_layout)
        self.new_password_label.setBuddy(self.new_password_lineedit)
        self.new_password_confirmation_label.setBuddy(self.new_password_confirmation_lineedit)
        self.current_password_label.setBuddy(self.current_password_lineedit)

        self.retranslateUi(PasswordChange)
        QtCore.QMetaObject.connectSlotsByName(PasswordChange)
        PasswordChange.setTabOrder(self.username_lineedit, self.current_password_lineedit)
        PasswordChange.setTabOrder(self.current_password_lineedit, self.new_password_lineedit)
        PasswordChange.setTabOrder(self.new_password_lineedit, self.new_password_confirmation_lineedit)

    def retranslateUi(self, PasswordChange):
        PasswordChange.setWindowTitle(QtGui.QApplication.translate("PasswordChange", "Change Password", None, QtGui.QApplication.UnicodeUTF8))
        self.username_label.setText(QtGui.QApplication.translate("PasswordChange", "Username:", None, QtGui.QApplication.UnicodeUTF8))
        self.new_password_label.setText(QtGui.QApplication.translate("PasswordChange", "New password:", None, QtGui.QApplication.UnicodeUTF8))
        self.new_password_confirmation_label.setText(QtGui.QApplication.translate("PasswordChange", "Re-enter new password:", None, QtGui.QApplication.UnicodeUTF8))
        self.current_password_label.setText(QtGui.QApplication.translate("PasswordChange", "Current password:", None, QtGui.QApplication.UnicodeUTF8))
        self.flash_label.setText(QtGui.QApplication.translate("PasswordChange", "<flash_label>", None, QtGui.QApplication.UnicodeUTF8))
        self.cancel_button.setText(QtGui.QApplication.translate("PasswordChange", "Close", None, QtGui.QApplication.UnicodeUTF8))
        self.ok_button.setText(QtGui.QApplication.translate("PasswordChange", "OK", None, QtGui.QApplication.UnicodeUTF8))

