# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'src/leap/bitmask/gui/ui/advanced_key_management.ui'
#
# Created: Thu Sep 18 13:04:05 2014
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_AdvancedKeyManagement(object):
    def setupUi(self, AdvancedKeyManagement):
        AdvancedKeyManagement.setObjectName("AdvancedKeyManagement")
        AdvancedKeyManagement.resize(504, 546)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/mask-icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        AdvancedKeyManagement.setWindowIcon(icon)
        self.gridLayout_4 = QtGui.QGridLayout(AdvancedKeyManagement)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.gbMyKeyPair = QtGui.QGroupBox(AdvancedKeyManagement)
        self.gbMyKeyPair.setObjectName("gbMyKeyPair")
        self.gridLayout_3 = QtGui.QGridLayout(self.gbMyKeyPair)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label = QtGui.QLabel(self.gbMyKeyPair)
        self.label.setObjectName("label")
        self.gridLayout_3.addWidget(self.label, 0, 0, 1, 1)
        self.leUser = QtGui.QLineEdit(self.gbMyKeyPair)
        self.leUser.setEnabled(True)
        self.leUser.setAlignment(QtCore.Qt.AlignCenter)
        self.leUser.setReadOnly(True)
        self.leUser.setObjectName("leUser")
        self.gridLayout_3.addWidget(self.leUser, 0, 1, 1, 1)
        self.label_3 = QtGui.QLabel(self.gbMyKeyPair)
        self.label_3.setObjectName("label_3")
        self.gridLayout_3.addWidget(self.label_3, 1, 0, 1, 1)
        self.leKeyID = QtGui.QLineEdit(self.gbMyKeyPair)
        self.leKeyID.setEnabled(True)
        self.leKeyID.setAlignment(QtCore.Qt.AlignCenter)
        self.leKeyID.setReadOnly(True)
        self.leKeyID.setObjectName("leKeyID")
        self.gridLayout_3.addWidget(self.leKeyID, 1, 1, 1, 1)
        self.label_5 = QtGui.QLabel(self.gbMyKeyPair)
        self.label_5.setObjectName("label_5")
        self.gridLayout_3.addWidget(self.label_5, 2, 0, 1, 1)
        self.leFingerprint = QtGui.QLineEdit(self.gbMyKeyPair)
        self.leFingerprint.setEnabled(True)
        self.leFingerprint.setAlignment(QtCore.Qt.AlignCenter)
        self.leFingerprint.setReadOnly(True)
        self.leFingerprint.setObjectName("leFingerprint")
        self.gridLayout_3.addWidget(self.leFingerprint, 2, 1, 1, 1)
        self.gridLayout = QtGui.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.pbExportKeys = QtGui.QPushButton(self.gbMyKeyPair)
        self.pbExportKeys.setObjectName("pbExportKeys")
        self.gridLayout.addWidget(self.pbExportKeys, 1, 1, 1, 1)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 1, 0, 2, 1)
        self.pbImportKeys = QtGui.QPushButton(self.gbMyKeyPair)
        self.pbImportKeys.setObjectName("pbImportKeys")
        self.gridLayout.addWidget(self.pbImportKeys, 2, 1, 1, 1)
        self.gridLayout_3.addLayout(self.gridLayout, 3, 0, 1, 2)
        self.gridLayout_4.addWidget(self.gbMyKeyPair, 1, 0, 1, 2)
        self.gbStoredPublicKeys = QtGui.QGroupBox(AdvancedKeyManagement)
        self.gbStoredPublicKeys.setObjectName("gbStoredPublicKeys")
        self.gridLayout_2 = QtGui.QGridLayout(self.gbStoredPublicKeys)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.twPublicKeys = QtGui.QTableWidget(self.gbStoredPublicKeys)
        self.twPublicKeys.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.twPublicKeys.setAlternatingRowColors(True)
        self.twPublicKeys.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.twPublicKeys.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.twPublicKeys.setTextElideMode(QtCore.Qt.ElideRight)
        self.twPublicKeys.setObjectName("twPublicKeys")
        self.twPublicKeys.setColumnCount(2)
        self.twPublicKeys.setRowCount(0)
        item = QtGui.QTableWidgetItem()
        self.twPublicKeys.setHorizontalHeaderItem(0, item)
        item = QtGui.QTableWidgetItem()
        self.twPublicKeys.setHorizontalHeaderItem(1, item)
        self.twPublicKeys.horizontalHeader().setStretchLastSection(True)
        self.gridLayout_2.addWidget(self.twPublicKeys, 0, 0, 1, 1)
        self.gridLayout_4.addWidget(self.gbStoredPublicKeys, 2, 0, 1, 2)
        self.lblStatus = QtGui.QLabel(AdvancedKeyManagement)
        self.lblStatus.setText("")
        self.lblStatus.setObjectName("lblStatus")
        self.gridLayout_4.addWidget(self.lblStatus, 3, 0, 1, 2)

        self.retranslateUi(AdvancedKeyManagement)
        QtCore.QMetaObject.connectSlotsByName(AdvancedKeyManagement)

    def retranslateUi(self, AdvancedKeyManagement):
        AdvancedKeyManagement.setWindowTitle(QtGui.QApplication.translate("AdvancedKeyManagement", "Advanced Key Management", None, QtGui.QApplication.UnicodeUTF8))
        self.gbMyKeyPair.setTitle(QtGui.QApplication.translate("AdvancedKeyManagement", "My key pair", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("AdvancedKeyManagement", "User:", None, QtGui.QApplication.UnicodeUTF8))
        self.leUser.setText(QtGui.QApplication.translate("AdvancedKeyManagement", "user_name@provider", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("AdvancedKeyManagement", "Key ID:", None, QtGui.QApplication.UnicodeUTF8))
        self.leKeyID.setText(QtGui.QApplication.translate("AdvancedKeyManagement", "key ID", None, QtGui.QApplication.UnicodeUTF8))
        self.label_5.setText(QtGui.QApplication.translate("AdvancedKeyManagement", "Key fingerprint:", None, QtGui.QApplication.UnicodeUTF8))
        self.leFingerprint.setText(QtGui.QApplication.translate("AdvancedKeyManagement", "fingerprint", None, QtGui.QApplication.UnicodeUTF8))
        self.pbExportKeys.setText(QtGui.QApplication.translate("AdvancedKeyManagement", "Export current key pair", None, QtGui.QApplication.UnicodeUTF8))
        self.pbImportKeys.setText(QtGui.QApplication.translate("AdvancedKeyManagement", "Import custom key pair", None, QtGui.QApplication.UnicodeUTF8))
        self.gbStoredPublicKeys.setTitle(QtGui.QApplication.translate("AdvancedKeyManagement", "Stored public keys", None, QtGui.QApplication.UnicodeUTF8))
        self.twPublicKeys.horizontalHeaderItem(0).setText(QtGui.QApplication.translate("AdvancedKeyManagement", "Email", None, QtGui.QApplication.UnicodeUTF8))
        self.twPublicKeys.horizontalHeaderItem(1).setText(QtGui.QApplication.translate("AdvancedKeyManagement", "Key ID", None, QtGui.QApplication.UnicodeUTF8))

import icons_rc
