# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'src/leap/bitmask/gui/ui/preferences_email_page.ui'
#
# Created: Mon Dec 29 09:53:14 2014
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_PreferencesEmailPage(object):
    def setupUi(self, PreferencesEmailPage):
        PreferencesEmailPage.setObjectName("PreferencesEmailPage")
        PreferencesEmailPage.resize(400, 300)

        self.retranslateUi(PreferencesEmailPage)
        QtCore.QMetaObject.connectSlotsByName(PreferencesEmailPage)

    def retranslateUi(self, PreferencesEmailPage):
        PreferencesEmailPage.setWindowTitle(QtGui.QApplication.translate("PreferencesEmailPage", "Form", None, QtGui.QApplication.UnicodeUTF8))

